/**
* Tip: een makkelijk manier om de database opnieuw te installeren op een device is om in webbeheer het versie nummer op te hogen (door een hoofdrubriek ofzo op te slaan).
* Dan zal het device bij de volgende start van het programma de nieuwe database data installeren.
*/

refwaarden.data.install = {

	apiurl : 'http://www.refwaarden.test.swis.nl/api.php',
	//apiurl : 'http://referentiewaarden.inaanbouw.nu/api.php',
	//apiurl : 'http://dev.local/referentiewaarden/api.php',

	db : null,
	geactiveerd : 0,
	localversion : 0,
	latestversion : 0,

	init : function(){
        var self = this;

        refwaarden.navigation.setState('loading'); // start with loading screen

		$(document).bind('deviceready', self.openDB()); // Wait for Cordova to load

		/*
		// controleren of de app is geactiveerd
		if(window.localStorage.getItem("activated") == 'true'){
			$(document).bind('deviceready', self.openDB()); // Wait for Cordova to load
        } else {
			refwaarden.navigation.setState('activate'); // activeer scherm
		}
		*/

    },

	// Database verbinding maken. Deze databaseverbinding wordt door de hele app gebruikt.
    openDB : function(){
    	var self = this;
    	self.db = window.openDatabase("refwaarden", "1.0", "Referentiewaarden DB", 5 * 1024 * 1024);
		self.db.transaction(self.createTables, self.errorCB, function(){self.createTablesSuccess()});
    },

	// Create the tables
	createTables : function (tx) {
		tx.executeSql('CREATE TABLE IF NOT EXISTS "activatie" ("id" INTEGER PRIMARY KEY  NOT NULL, "geactiveerd" INTEGER NOT NULL)');
	    tx.executeSql('CREATE TABLE IF NOT EXISTS "favorites" ("id" INTEGER PRIMARY KEY  NOT NULL  UNIQUE)');
		tx.executeSql('CREATE TABLE IF NOT EXISTS "hoofdrubriek" ("id" INTEGER PRIMARY KEY  NOT NULL, "titel" VARCHAR NOT NULL )');
		tx.executeSql('CREATE TABLE IF NOT EXISTS "notities" ("hoofdrubriek" INTEGER PRIMARY KEY  NOT NULL, "tekst" TEXT NOT NULL )');
	    tx.executeSql('CREATE TABLE IF NOT EXISTS "emailteksten" ("id" INTEGER PRIMARY KEY  NOT NULL, "subject" VARCHAR NOT NULL , "body" TEXT NOT NULL )');
	    tx.executeSql('CREATE TABLE IF NOT EXISTS "paginateksten" ("id" INTEGER PRIMARY KEY  NOT NULL, "titel" VARCHAR NOT NULL , "titel_key" VARCHAR NOT NULL , "volgorde" FLOAT NOT NULL , "tekst" TEXT NOT NULL )');
	    tx.executeSql('CREATE TABLE IF NOT EXISTS "referentiewaarde" ("id" INTEGER PRIMARY KEY  NOT NULL, "hoofdrubriek" INTEGER NOT NULL, "titel" VARCHAR, "volgorde" FLOAT NOT NULL , "referentiewaarde" VARCHAR NOT NULL )');
		tx.executeSql('CREATE TABLE IF NOT EXISTS "versie" ("id" INTEGER PRIMARY KEY  NOT NULL, "versie" INTEGER NOT NULL)');
	},

    // Transaction error callback
	errorCB : function (err) {
	    alert("Error processing SQL: "+err.code + " " + err.message);
	},

	// Transaction success callback
	createTablesSuccess : function () {
		var self = this;

		// controleren of de app al is geactiveerd
		self.db.transaction(function(tx){
			tx.executeSql('SELECT geactiveerd FROM activatie', [], function(tx, results){
				// als er een rij is gevonden, betekend dat dat de app geactiveerd is. Anders is geactiveerd gelijk aan 0.
				if(results.rows.length > 0){
					self.geactiveerd = results.rows.item(0).geactiveerd;
				}

				if(self.geactiveerd == 1){
					// door met het vullen van de database
					self.populateTables();
				} else {
					// doorsturen naar activatiescherm
					refwaarden.navigation.setState('activate'); // activeer scherm
				}

			}, function(err){self.errorCB(err)});
		}, self.errorCB);


	},

	// Database vullen met online data (indien de gebruiker online is en een verouderde database heeft) of met lokale data (indien de gebruiker offline is en nog geen data heeft in zijn database)
	populateTables : function(){
		var self = this;
		self.db.transaction(function(tx){self.populateTablesNow(tx)}, self.errorCB);
	},

	populateTablesNow : function(tx) {

		var self = this;

		// versie nummer van de lokale database opvragen
		tx.executeSql('SELECT versie FROM versie', [], function(tx, results){

			// als er een rij is gevonden, betekend dat dat de lokale database al een keer is gevuld. Anders is localversion gelijk aan 0.
			if(results.rows.length > 0){
				self.localversion = results.rows.item(0).versie;
			}

			// checken wat de laatste versie is (online)
			if(self.isOnline){
				$.getJSON(self.apiurl, { request: "version" }, function(json) {
					self.latestversion = json.versie;

		    		if(parseInt(self.localversion,10) < parseInt(self.latestversion,10)){

						// nieuwste data binnenhalen
						$.getJSON(self.apiurl, { request: "update" }, function(json) {
							// nieuwste data in de lokale database zetten
							//console.log("online data installeren");
							self.insertData(json);
					    });

		    		} else {
		    			//console.log("isOnline: geen update nodig");
                        $(window).trigger('dbready');
		    		}
			    });
			} else {
				if(self.localversion == 0){
					// lokale data installeren
					//console.log("lokale data installeren");
					self.insertData(refwaarden.data.localdatabase);
				} else {
					//console.log("isOffline: geen update nodig");
					$(window).trigger('dbready');
		    	}
			}

		}, function(err){self.errorCB(err)});

	},


	// deze functie kijkt of de telefoon verbonden is met internet
	isOnline : function(){
		if(navigator.network.connection.type != Connection.NONE){
			return true;
		} else {
			return false;
		};
	},

	// deze functie vult a.d.v. een json object de lokale database
	insertData : function(jsondata){
		var self = this;

		self.db.transaction(
			function (tx) {
				// Alle lokale tabellen leegmaken, zodat deze gevuld kunnen worden met de nieuwe data (TRUNCATE werkt niet in Sqlite)
				tx.executeSql('DELETE FROM hoofdrubriek');
				tx.executeSql('DELETE FROM referentiewaarde');
				tx.executeSql('DELETE FROM emailteksten');
				tx.executeSql('DELETE FROM paginateksten');
				tx.executeSql('DELETE FROM versie');
			},
			self.errorCB,
			function(){
				// success functie

				// alle data inserten
				self.db.transaction(
					function (tx) {

						// hoofdrubriek
						for(i=0; i<jsondata.hoofdrubriek.length; i++){
							tx.executeSql('INSERT INTO hoofdrubriek (id, titel) VALUES (?, ?)', [jsondata.hoofdrubriek[i].id, jsondata.hoofdrubriek[i].titel]);
						}

						// emailteksten
						for(i=0; i<jsondata.emailteksten.length; i++){
							tx.executeSql('INSERT INTO emailteksten (id, subject, body) VALUES (?, ?, ?)', [jsondata.emailteksten[i].id, jsondata.emailteksten[i].subject, jsondata.emailteksten[i].body]);
						}

						// paginateksten
						for(i=0; i<jsondata.paginateksten.length; i++){
							tx.executeSql('INSERT INTO paginateksten (id, titel, titel_key, volgorde, tekst) VALUES (?, ?, ?, ?, ?)', [jsondata.paginateksten[i].id, jsondata.paginateksten[i].titel, jsondata.paginateksten[i].titel_key, jsondata.paginateksten[i].volgorde, jsondata.paginateksten[i].tekst]);
						}

						// referentiewaarde
						for(i=0; i<jsondata.referentiewaarde.length; i++){
							tx.executeSql('INSERT INTO referentiewaarde (id, hoofdrubriek, titel, volgorde, referentiewaarde) VALUES (?, ?, ?, ?, ?)', [jsondata.referentiewaarde[i].id, jsondata.referentiewaarde[i].hoofdrubriek, jsondata.referentiewaarde[i].titel, jsondata.referentiewaarde[i].volgorde, jsondata.referentiewaarde[i].referentiewaarde]);
						}

						// versie (deze tabel altijd als laatste vullen!!! -> mocht er in de voorgaande acties iets misgaan, dan wordt bij een herstart van de app een nieuwe versie geinstalleerd)
						for(i=0; i<jsondata.versie.length; i++){
							tx.executeSql('INSERT INTO versie (id, versie) VALUES (?, ?)', [jsondata.versie[i].id, jsondata.versie[i].versie]);
						}

					},
					self.errorCB,
					function(){
						// success functie
						$(window).trigger('dbready');
					}
				);
			}
		);
	}
};

